@@ -342,7 +342,7 @@ byteM_parse_codeset(
 	    continue;
 
         for (j = 0; j < codeset->length; j++) {
-	    ch = *((unsigned char *)(inbufptr + j));
+	    ch = *((const unsigned char *)(inbufptr + j));
 	    byteM_rec = byteM[j];
 	    byteinfo = byteM_rec.byteinfo;
 
@@ -764,9 +764,6 @@ mbstowcs_org(
         return( 0 );
     }
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
 	ch = *inbufptr++;
@@ -921,9 +918,6 @@ wcstombs_org(
     int defstr_len = strlen(default_string);
 
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -1080,9 +1074,6 @@ wcstocts(
     int from_size = *from_left;
     char *ext_seg_len = NULL;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -1233,7 +1224,7 @@ stdc_wcstocts(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1272,9 +1263,6 @@ ctstowcs(
         return( 0 );
     }
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
 	ch = *inbufptr++;
@@ -1556,7 +1544,7 @@ stdc_ctstowcs(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1592,7 +1580,7 @@ stdc_cstowcs(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1628,7 +1616,7 @@ mbstocts(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -1666,9 +1654,6 @@ mbstostr(
         return( 0 );
     }
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
 	ch = *inbufptr++;
@@ -1956,10 +1941,6 @@ wcstostr(
     const char *default_string = XLC_PUBLIC(lcd, default_string);
     int defstr_len = strlen(default_string);
 
-
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -2082,7 +2063,7 @@ stdc_wcstostr(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2114,9 +2095,6 @@ wctocs(
     char *outbufptr = *to;
     int from_size = *from_left;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     if (*from_left && *to_left) {
 
         wc = *inbufptr++;
@@ -2360,7 +2338,7 @@ ctstombs(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2396,7 +2374,7 @@ cstombs(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2427,9 +2405,6 @@ strtombs(
     char *outbufptr = *to;
     int from_size = *from_left;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         ch = *inbufptr++;
@@ -2537,9 +2512,6 @@ strtowcs(
     wchar_t *outbufptr = (wchar_t *)*to;
     int from_size = *from_left;
 
-    if (*from_left > *to_left)
-        *from_left = *to_left;
-
     while (*from_left && *to_left) {
 
         ch = *inbufptr++;
@@ -2611,7 +2583,7 @@ stdc_strtowcs(
 
 ret:
     if (buf)
-	Xfree((char *)buf);
+	Xfree(buf);
 
     return (unconv_num1 + unconv_num2);
 }
@@ -2625,14 +2597,14 @@ close_converter(
     XlcConv conv)
 {
     if (conv->state) {
-	Xfree((char *) conv->state);
+	Xfree(conv->state);
     }
 
     if (conv->methods) {
-	Xfree((char *) conv->methods);
+	Xfree(conv->methods);
     }
 
-    Xfree((char *) conv);
+    Xfree(conv);
 }
 
 /* -------------------------------------------------------------------------- */
@@ -2647,11 +2619,11 @@ create_conv(
     XlcConv conv;
     State state;
 
-    conv = (XlcConv) Xcalloc(1, sizeof(XlcConvRec));
+    conv = Xcalloc(1, sizeof(XlcConvRec));
     if (conv == NULL)
 	return (XlcConv) NULL;
 
-    conv->methods = (XlcConvMethods) Xmalloc(sizeof(XlcConvMethodsRec));
+    conv->methods = Xmalloc(sizeof(XlcConvMethodsRec));
     if (conv->methods == NULL)
 	goto err;
     *conv->methods = *methods;
