@@ -57,7 +57,7 @@ char **XGetFontPath(
 	    }
 
 	    if ((! flist) || (! ch)) {
-		if (flist) Xfree((char *) flist);
+		if (flist) Xfree(flist);
 		if (ch) Xfree(ch);
 		_XEatDataWords(dpy, rep.length);
 		UnlockDisplay(dpy);
@@ -93,7 +93,7 @@ XFreeFontPath (char **list)
 {
 	if (list != NULL) {
 		Xfree (list[0]-1);
-		Xfree ((char *)list);
+		Xfree (list);
 	}
 	return 1;
 }
